<?php

namespace FSPoster\App\Providers\WPPost;

use FSPoster\App\Models\Schedule;
use FSPoster\App\Providers\Core\Settings;
use FSPoster\App\Providers\Helpers\Helper;
use FSPoster\App\Providers\Schedules\ScheduleService;
use WP_Post;

class WPPostService
{

	public static function setObjectTerms ( $wpPostId, $terms, $tt_ids, $taxonomy, $append, $old_tt_ids )
	{
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
			return;

		if ( wp_is_post_autosave( $wpPostId ) || wp_is_post_revision( $wpPostId ) )
			return;

		$termsUpdated = ! empty( array_diff( $old_tt_ids, $tt_ids ) ) || ! empty( array_diff( $tt_ids, $old_tt_ids ) );

		if( $termsUpdated )
			self::postMutation( get_post( $wpPostId ), false, false, true );
	}

	public static function postUpdated ( $wpPostId, $postAfter, $postBefore )
	{
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
			return;

		if ( wp_is_post_autosave( $wpPostId ) || wp_is_post_revision( $wpPostId ) )
			return;

		$postDateChanged   = $postBefore->post_date != $postAfter->post_date;
		$postStatusChanged = ScheduleService::getScheduleStatusByWpPostStatus( $postBefore->post_status ) != ScheduleService::getScheduleStatusByWpPostStatus( $postAfter->post_status );

		if ( $postStatusChanged || $postDateChanged )
			self::postMutation( $postAfter, $postStatusChanged, $postDateChanged, false );
	}

    public static function postMutation ( WP_Post $post, $postStatucChanged, $postDateChanged, $postTermsChanged )
    {
		if( ! $postStatucChanged && ! $postDateChanged && ! $postTermsChanged )
			return;

        // if post created by FS Poster (calendar), then this hook is not needed
        if ( $post->post_type === 'fsp_post' )
            return;

	    if ( !in_array( $post->post_status, [ 'publish', 'future', 'draft', 'pending' ] ) )
		    return;

        // if post type is not whitelisted, just skip
        if ( !in_array( $post->post_type, Settings::get( 'allowed_post_types', [ 'post', 'page', 'attachment', 'product' ] ) ) )
            return;

	    $wpPostId = $post->ID;

	    if( metadata_exists( 'post', $wpPostId, 'fsp_runned_for_this_post' ) )
		    return;

	    if( metadata_exists( 'post', $wpPostId, 'fsp_enable_auto_share' ) )
		    $autoShareOn = (bool)get_post_meta( $wpPostId, 'fsp_enable_auto_share', true );
		else
			$autoShareOn = (bool)Settings::get( 'auto_share', true );

	    $scheduleGroupId = get_post_meta( $wpPostId, 'fsp_schedule_group_id', true );

		if( ! empty( $scheduleGroupId ) )
		{
			$schedulesCreatedByUserAction = get_post_meta( $wpPostId, 'fsp_schedule_created_manually', true );
			$needToRecreateSchedules = ! $schedulesCreatedByUserAction && $postTermsChanged;
			$needToUpdateSchedulesDateAndStatus = $postStatucChanged || $postDateChanged;

			if( ! $autoShareOn && $post->post_status == 'publish' )
			{
				ScheduleService::deleteScheduleCacheDataForWpPost( $wpPostId );
			}
			else if( $needToRecreateSchedules )
			{
				ScheduleService::deleteSchedulesFromWpPost( $wpPostId, $scheduleGroupId );
				ScheduleService::createSchedulesFromWpPost( $wpPostId, $scheduleGroupId );
			}
			else if( $needToUpdateSchedulesDateAndStatus )
			{
				ScheduleService::updateSchedulesStatusAndDeteFromWpPost( $wpPostId, $scheduleGroupId );
			}
		}
		else
		{
			if( $autoShareOn )
			{
				$schedulesGroupId = ScheduleService::createSchedulesFromWpPost( $wpPostId );
				update_post_meta( $wpPostId, 'fsp_schedule_group_id', $schedulesGroupId );
			}

			/*
			 * Auto-share checkboxunun metasini yaradir. Ola biler ki, checkbox deyishdirilmeyibdi (enable/disable edilmeyib).
			 * Checkbox basilmayib deye Ajax requestde getmeyib backende ki, bu meta create edilsin.
			 * Ona gore de WP save hooknda default settingsdek durumu metaya yazdiririg.
			*/
			update_post_meta( $wpPostId, 'fsp_enable_auto_share', $autoShareOn ? 1 : 0 );
		}
    }

    public static function deletePostSchedules ( $postId )
    {
        Schedule::where( 'blog_id', Helper::getBlogId() )->where( 'wp_post_id', $postId )->delete();
    }

}