<?php

namespace FSPoster\App\SocialNetworks\Threads\Adapters;

use FSPoster\App\Models\Channel;
use FSPoster\App\Providers\Channels\ChannelService;
use FSPoster\App\SocialNetworks\Threads\Api\Api;
use FSPoster\App\SocialNetworks\Threads\App\Bootstrap;

class ChannelAdapter
{

    public static function fetchChannels ( $api, $method = 'login_pass' ): array
    {
	    $me = $api->getMyInfo();

		$profileName = $me['name'] ?: $api->authData->username;
        $username = $me['username'] ?: $api->authData->username;

	    $channelSessionId = ChannelService::addChannelSession( [
		    'name'           => $profileName,
		    'social_network' => Bootstrap::getInstance()->getSlug(),
		    'remote_id'      => $api->authData->userId,
		    'proxy'          => $api->proxy ?? '',
		    'method'         => $method,
		    'data'           => [
			    'auth_data' => (array)$api->authData
		    ]
	    ] );

	    $existingChannels = Channel::where( 'channel_session_id', $channelSessionId )
	                               ->select( [ 'id', 'remote_id', 'channel_type' ], true )
	                               ->fetchAll();

	    $existingChannelsIdToRemoteIdMap = [];

	    foreach ( $existingChannels as $existingChannel )
	    {
		    $existingChannelsIdToRemoteIdMap[ $existingChannel->remote_id . ':' . $existingChannel->channel_type ] = $existingChannel->id;
	    }

	    $channelsList = [];
	    $channelsList[] = [
		    'id'                    => $existingChannelsIdToRemoteIdMap[ $api->authData->userId ] ?? null,
		    'social_network'        => 'threads',
		    'name'                  => $profileName,
		    'channel_type'          => 'account',
		    'remote_id'             => $api->authData->userId,
		    'picture'               => $me['profile_pic'],
		    'channel_session_id'    => $channelSessionId,
		    'data'                  => [
				'username'  =>  $username
		    ]
	    ];

	    return $channelsList;
    }

}