<?php

namespace FSPoster\App\SocialNetworks\Threads\Api;

use Exception;
use FSPoster\App\Providers\Helpers\GuzzleClient;
use FSPoster\App\Providers\Helpers\Helper;
use FSPoster\GuzzleHttp\Cookie\SetCookie;

class Api
{

	public AuthData $authData;
	public ?string  $proxy = null;

	public string $authException = \Exception::class;
	public string $postException = \Exception::class;

	public function setProxy ( ?string $proxy ): self
	{
		$this->proxy = $proxy;

		return $this;
	}

	public function setAuthData ( AuthData $authData ): self
	{
		$this->authData = $authData;

		return $this;
	}

	public function setAuthException ( string $exceptionClass ): self
	{
		$this->authException = $exceptionClass;

		return $this;
	}

	public function setPostException ( string $exceptionClass ): self
	{
		$this->postException = $exceptionClass;

		return $this;
	}

    public function sendPost ( PostingData $postingData ): array
    {
        $postURL  = 'https://www.threads.net/api/v1/media/configure_text_only_post/';
        $uploadID = (string)(int)( microtime( true ) * 1000 );

        $data = [
            'caption'            => $postingData->message,
            'publish_mode'       => 'text_post',
            'text_post_app_info' => [ 'reply_control' => 0 ],
            'upload_id'          => $uploadID,
        ];

        if ( ! empty( $postingData->uploadMedia ) )
        {
            $this->uploadIgPhoto( $uploadID, $postingData->uploadMedia[0]['path'] );

	        $postURL = 'https://www.threads.net/api/v1/media/configure_text_post_app_feed/';
            $data['scene_capture_type'] = '';
            unset( $data['publish_mode'] );
        }
		else if ( ! empty( $postingData->link ) )
		{
			$data['text_post_app_info']['link_attachment_url'] = $postingData->link;
		}

        $data['text_post_app_info'] = json_encode( $data['text_post_app_info'] );

        try
        {
            $response = $this->getClient()->post( $postURL, [ 'form_params' => $data ] )->getBody()->getContents();
        } catch ( Exception $e )
        {
            throw new $this->postException( $e->getMessage() );
        }

        $response = json_decode( $response, true );

        if ( ! isset( $response['media']['code'] ) )
	        throw new $this->postException( $response['message'] ?? fsp__( 'Unknown error!' ) );

        return [
            'id' => (string)$response['media']['code']
        ];
    }

    private function uploadIgPhoto ( $uploadId, $photo )
    {
        $params = [
            'is_sidecar' => '0',
            'is_threads' => '1',
            'media_type' => '1',
            'upload_id'  => $uploadId,
        ];

        $entity_name = sprintf( 'fb_uploader_%d', $uploadId );
        $endpoint    = 'https://www.threads.net/rupload_igphoto/' . $entity_name;

        try
        {
            $response = (string)$this->getClient()->post( $endpoint, [
                'headers' => [
                    'X-Instagram-Rupload-Params' => json_encode( $params ),
                    'X-Entity-Type'              => Helper::mimeContentType( $photo ),
                    'X-Entity-Name'              => $entity_name,
                    'X-Entity-Length'            => filesize( $photo ),
                    'Offset'                     => '0',
                ],
                'body'    => file_get_contents( $photo ),
            ] )->getBody();

            $response = json_decode( $response, true );
        } catch ( Exception $e )
        {
	        throw new $this->postException( $e->getMessage() );
        }

        return $response;
    }

    public function login (): array
    {
        /**
         * :0: = pass plaintext formada olsun (encrypt olmasin)
         */
        $encPass = '#PWD_INSTAGRAM_BROWSER:0:' . time() . ':' . $this->authData->password;

        $data = [
            'enc_password' => $encPass,
            'username'     => $this->authData->username,
        ];

        try
        {
            $resp = $this->getClient()->post( 'https://www.threads.net/api/v1/web/accounts/login/ajax/', [ 'form_params' => $data ] );
        } catch ( Exception $e )
        {
            throw new $this->authException( $e->getMessage() );
        }

	    $respArr = json_decode( $resp->getBody()->getContents(), true );

	    /*if( empty( $respArr['logged_in_user']['text_post_app_joiner_number'] ) )
		    throw new $this->authException( fsp__( 'User have not joined Threads yet!' ) );*/

	    $status = $body['status'] ?? 'fail';
		$userId = $respArr['userId'] ?? '';
		$sessionId = $this->getCookieFromResponse( $resp, 'sessionid' );

	    if ( ( $respArr['two_factor_required'] ?? false ) === true && isset( $respArr['two_factor_info'] ) )
		    return $this->twoFactorData( $resp );

		if ( $status !== 'ok' || ( $respArr['authenticated'] ?? false ) !== true || empty( $userId ) || empty( $sessionId ) )
			throw new $this->authException( $respArr[ 'message' ] ?? fsp__( 'Login failed!' ) );

	    $this->authData->userId    = $userId;
	    $this->authData->sessionId = $sessionId;

	    return [ 'needs_challenge' => false ];
    }

    public function doTwoFactorAuth ( $two_factor_identifier, $code, $verification_method = '1' ): bool
    {
        $code = preg_replace( '/\s+/', '', $code );

        $data = [
            'identifier'          => $two_factor_identifier,
            'verificationCode'    => $code,
            'verification_method' => $verification_method,
            'username'            => $this->authData->username,
        ];

        try
        {
            $resp = $this->getClient()->post( 'https://www.threads.net/api/v1/web/accounts/login/ajax/two_factor/', [ 'form_params' => $data ] );
        } catch ( Exception $e )
        {
            throw new $this->authException( $e->getMessage() );
        }

	    $body = json_decode( $resp->getBody(), true );

	    $status = $body['status'] ?? 'fail';

	    if ( $status !== 'ok' || ( $body['authenticated'] ?? false ) !== true || empty( $body['userId'] ) )
	        throw new $this->authException( $body['message'] ?? fsp__( '2FA failed!' ) );

	    $this->authData->userId    = $body['userId'];
	    $this->authData->sessionId = $this->getCookieFromResponse( $resp, 'sessionid' );

		return true;
    }

    public function getMyInfo ()
    {
	    $fullName      = '';
	    $profilePicUrl = '';

	    try
	    {
		    $userBio = (string)$this->getClient()->get( 'https://www.threads.net/@' . urlencode( $this->authData->username ), [
			    'allow_redirects' => false,
			    'headers'         => [ 'Sec-Fetch-Mode' => 'navigate' ],
		    ] )->getBody();
	    } catch ( Exception $e )
	    {
		    throw new $this->authException( $e->getMessage() );
	    }

	    preg_match( '/\"full_name\":(\"[^\"]*\")/', $userBio, $fName );
	    if ( isset( $fName[1] ) )
		    $fullName = json_decode( $fName[1] );

	    preg_match( '/\"profile_pic_url\":(\"[^\"]*\")/', $userBio, $pic );
	    if ( isset( $pic[1] ) )
		    $profilePicUrl = json_decode( $pic[1] );

        return [
            'name'        => $fullName,
            'profile_pic' => $profilePicUrl
        ];
    }

    private function twoFactorData ( $response )
    {
        $body = json_decode( $response->getBody(), true );

        $verification_method = '1';

        if ( $body['two_factor_info']['whatsapp_two_factor_on'] )
            $verification_method = '6';

        if ( $body['two_factor_info']['totp_two_factor_on'] )
            $verification_method = '3';

		$this->authData->mid = $this->getCookieFromResponse( $response, 'mid' );

        return [
            'needs_challenge' => true,
            'options'         => [
                'auth_data'                 => (array)$this->authData,
                'verification_method'       => $verification_method,
                'two_factor_identifier'     => $body['two_factor_info']['two_factor_identifier'],
                'obfuscated_phone_number'   => $body['two_factor_info']['obfuscated_phone_number'] ?? ( $body['two_factor_info']['obfuscated_phone_number_2'] ?? '' ),
            ]
        ];
    }

    private function getCookieFromResponse ( $response, $cookieName )
    {
        $headerSetCookies = $response->getHeader( 'Set-Cookie' );

        foreach ( $headerSetCookies as $header )
        {
            $cookie = SetCookie::fromString( $header );

            if ( $cookie->getName() === $cookieName )
                return $cookie->getValue();
        }

        return '-';
    }

    private function getClient ()
    {
        $csrfToken = uniqid();

        $cookie = 'csrftoken=' . $csrfToken . ';';

        if ( !empty( $this->authData->userId ) )
            $cookie .= 'ds_user_id=' . $this->authData->userId . ';';

        if ( !empty( $this->authData->sessionId ) )
            $cookie .= 'sessionid=' . $this->authData->sessionId . ';';

        if ( !empty( $this->authData->mid ) )
            $cookie .= 'mid=' . $this->authData->mid . ';';

        return new GuzzleClient( [
            'verify'  => false,
            'headers' => [
                'User-Agent'    => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:124.0) Gecko/20100101 Firefox/124.0',
                'X-csrftoken'   => $csrfToken,
                'X-IG-APP-ID'   => '238260118697367',
                'Cookie'        => $cookie,
	            'Accept'        => 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/png,image/svg+xml,*/*;q=0.8'
            ],
            'proxy'   => empty( $this->proxy ) ? null : $this->proxy,
        ] );
    }

}