<?php

namespace FSPoster\App\SocialNetworks\Threads\App;

use Exception;
use FSPoster\App\Providers\Channels\ChannelSessionException;
use FSPoster\App\Providers\Core\RestRequest;
use FSPoster\App\Providers\Core\Settings;
use FSPoster\App\SocialNetworks\Threads\Adapters\ChannelAdapter;
use FSPoster\App\SocialNetworks\Threads\Api\Api;
use FSPoster\App\SocialNetworks\Threads\Api\AuthData;

class Controller
{
    /**
     * @throws Exception
     */
    public static function addChannel ( RestRequest $request ): array
    {
        $username = $request->require( 'username', RestRequest::TYPE_STRING, fsp__( 'Please enter the username' ) );
        $password = $request->require( 'password', RestRequest::TYPE_STRING, fsp__( 'Please enter the password' ) );
        $proxy    = $request->param( 'proxy', '', RestRequest::TYPE_STRING );

	    $authData = new AuthData();
	    $authData->username = $username;
		$authData->password = $password;

		$api = new Api();
		$api->setProxy( $proxy )
			->setAuthException( ChannelSessionException::class )
			->setAuthData( $authData );

        $result = $api->login();

        if ( $result['needs_challenge'] )
            return [ 'options' => $result[ 'options' ] ];

		$channels = ChannelAdapter::fetchChannels( $api );

        return [ 'channels' => $channels ];
    }

    public static function doChallenge ( RestRequest $request ): array
    {
        $options = $request->require( 'options', RestRequest::TYPE_ARRAY, fsp__( 'Options are empty' ) );
        $code    = $request->require( 'code', RestRequest::TYPE_STRING, fsp__( 'Please enter the code' ) );
        $proxy   = $request->param( 'proxy', '', RestRequest::TYPE_STRING );

	    if( empty( $options['auth_data'] ) || ! is_array( $options['auth_data'] ) )
		    throw new Exception( fsp__( 'Something went wrong!' ) );

	    $authData = new AuthData();
	    $authData->setFromArray( $options['auth_data'] );

	    $api = new Api();
	    $api->setProxy( $proxy )
	        ->setAuthException( ChannelSessionException::class )
	        ->setAuthData( $authData );

        $api->doTwoFactorAuth( $options[ 'two_factor_identifier' ], $code, $options[ 'verification_method' ] );

	    $channels = ChannelAdapter::fetchChannels( $api );

	    return [ 'channels' => $channels ];
    }

    public static function saveSettings ( RestRequest $request ): array
    {
        $postText           = $request->param( 'post_text', '', RestRequest::TYPE_STRING );
        $cutPostText        = (int)$request->param( 'cut_post_text', false, RestRequest::TYPE_BOOL );
	    $attachLink         = $request->param( 'attach_link', true, RestRequest::TYPE_BOOL );
	    $uploadMedia        = $request->param( 'upload_media', false, RestRequest::TYPE_BOOL );
	    $mediaTypeToUpload  = $request->param( 'media_type_to_upload', 'featured_image', RestRequest::TYPE_STRING );

        Settings::set( 'threads_post_content', $postText );
        Settings::set( 'threads_cut_post_text', $cutPostText );
	    Settings::set( 'threads_attach_link', (int)$attachLink );
	    Settings::set( 'threads_upload_media', (int)$uploadMedia );
	    Settings::set( 'threads_media_type_to_upload', $mediaTypeToUpload );

	    do_action( 'fsp_save_settings', $request, Bootstrap::getInstance()->getSlug() );

        return [];
    }

    public static function getSettings ( RestRequest $request ): array
    {
	    return apply_filters('fsp_get_settings', [
		    'post_text'            => Settings::get( 'threads_post_content', '{post_title}' ),
		    'cut_post_text'        => (bool)Settings::get( 'threads_cut_post_text', true ),
		    'attach_link'           => (bool)Settings::get( 'threads_attach_link', true ),
		    'upload_media'          => (bool)Settings::get( 'threads_upload_media', false ),
		    'media_type_to_upload'  => Settings::get( 'threads_media_type_to_upload', 'featured_image' )
	    ], Bootstrap::getInstance()->getSlug());
    }

}